﻿using System;
using System.Text;

namespace TailIsland.JisX0208TableGenerator
{
    public static class Program
    {
        public static void Main(string[] args)
        {
            string enableChars = GetAsciiAndJisX0208AndMicorosftErrorChars();

            for (int c = 0x0000; c <= 0xffff; )
            {
                Console.Write("                ");

                for (int i = 0; i < 32; ++i)
                {
                    byte b = 0x00;

                    for (int j = 0; j < 8; ++j)
                    {
                        if (enableChars.Contains(Convert.ToString((char)c++)))
                        {
                            b |= (byte)(0x01 << j);
                        }
                    }

                    Console.Write("0x{0:x2}, ", b);
                }

                Console.WriteLine("  //  0x{0:X4}", c - 32 * 8);
            }
        }

        private static string GetAsciiAndJisX0208AndMicorosftErrorChars()
        {
            StringBuilder sb = new StringBuilder();

            sb.Append(GetControlChars());
            sb.Append(GetAsciiChars());
            sb.Append(GetJisX0208Chars());
            sb.Append(GetMicrosoftErrorChars());

            return sb.ToString();
        }

        private static string GetControlChars()
        {
            StringBuilder sb = new StringBuilder();

            sb.Append('\t');
            sb.Append('\r');
            sb.Append('\n');

            return sb.ToString();
        }

        private static string GetAsciiChars()
        {
            StringBuilder sb = new StringBuilder();

            for (int c = 0x0020; c <= 0x007e; ++c)
            {
                sb.Append(GetAsciiChar(c));
            }

            return sb.ToString();
        }

        private static char GetAsciiChar(int asciiCode)
        {
            return (char)asciiCode;
        }

        private static string GetJisX0208Chars()
        {
            StringBuilder sb = new StringBuilder();

            for (int h = 0x0021; h <= 0x0074; ++h)
            {
                if (h >= 0x0029 && h <= 0x002f)
                {
                    continue;
                }

                for (int l = 0x0020; l < 0x007f; ++l)
                {
                    sb.Append(GetJisX0208Char(h << 8 | l));
                }
            }

            return sb.ToString();
        }

        private static char GetJisX0208Char(int jisCode)
        {
            return Encoding.GetEncoding("iso-2022-jp").GetChars(new byte[] { 0x1b, 0x24, 0x42, (byte)(jisCode >> 8), (byte)(jisCode & 0x00ff), 0x1b, 0x28 })[0];
        }

        private static string GetMicrosoftErrorChars()
        {
            StringBuilder sb = new StringBuilder();

            sb.Append('\u00a2');  //  CENT SIGN問題への対応。
            sb.Append('\u00a3');  //  POUND SIGN問題への対応。
            sb.Append('\u00ac');   //  NOT SIGN問題への対応。
            sb.Append('\u2014');  //  EM DASH問題への対応。
            sb.Append('\u2016');  //  DOUBLE VERTICAL LINE問題への対応。
            sb.Append('\u2212');  //  MINUS SIGN問題への対応。
            sb.Append('\u301c');  //  WAVE DASH問題への対応。

            return sb.ToString();
        }
    }
}
