﻿if object_id(N'OrderNumber', 'U') is not null drop table [OrderNumber]
go

if object_id(N'ReservedItem', 'U') is not null drop table [ReservedItem]
go

if object_id(N'Photograph', 'U') is not null drop table [Photograph]
go

if object_id(N'OrderDetail', 'U') is not null drop table [OrderDetail]
go

if object_id(N'Order', 'U') is not null drop table [Order]
go

if object_id(N'UserInformation', 'U') is not null drop table [UserInformation]
go

if object_id(N'Item', 'U') is not null drop table [Item]
go

if object_id(N'Product', 'U') is not null drop table [Product]
go

if object_id(N'Category', 'U') is not null drop table [Category]
go

create table [Category]
(
	[Identity] bigint not null,
	[Name] nvarchar(64) not null,
	[Description] nvarchar(256) not null,
	
	constraint [Category_PK] primary key ([Identity])
)
go

create table [Product]
(
	[Identity] bigint not null,
	[CategoryIdentity] bigint not null,
	[Name] nvarchar(64) not null,
	[Description] nvarchar(256) not null,
	[PhotographNumber] uniqueidentifier not null,
	
	constraint [Product_PK] primary key ([Identity]),
	constraint [Product_Category_FK] foreign key ([CategoryIdentity]) references [Category]([Identity])
)
go

create table [Item]
(
	[Identity] bigint not null,
	[ProductIdentity] bigint not null,
	[Name] nvarchar(64) not null,
	[Price] decimal(10, 2) not null,
	[Quantity] int not null,
	
	constraint [Item_PK] primary key ([Identity]),
	constraint [Item_Product_FK] foreign key ([ProductIdentity]) references [Product]([Identity])
)
go

create table [UserInformation]
(
	[UserInformationNumber] uniqueidentifier not null,
	[ZipCode] nvarchar(20) not null,
	[Address1] nvarchar(64) not null,
	[Address2] nvarchar(64) not null,
	[PhoneNumber] nvarchar(20) not null,
	[Name] nvarchar(64) not null,
	
	constraint [UserInformation_PK] primary key ([UserInformationNumber])
)
go

create table [Order]
(
	[OrderNumber] nchar(7) not null,
	[UserInformationNumber] uniqueidentifier not null,
	[ShippingZipCode] nvarchar(20) not null,
	[ShippingAddress1] nvarchar(64) not null,
	[ShippingAddress2] nvarchar(64) not null,
	[ShippingPhoneNumber] nvarchar(20) not null,
	[ShippingName] nvarchar(64) not null,
	[CreditCardChargeNumber] nvarchar(20) not null,
	
	constraint [Order_PK] primary key ([OrderNumber]),
	constraint [Order_UserInformation_FK] foreign key ([UserInformationNumber]) references [UserInformation]([UserInformationNumber])
)
go

create table [OrderDetail]
(
	[Identity] bigint not null,
	[OrderNumber] nchar(7) not null,
	[ItemIdentity] bigint not null,
	[Price] decimal(10, 2) not null,
	[Quantity] int not null,
	
	constraint [OrderDetail_PK] primary key ([Identity]),
	constraint [OrderDetail_Order_FK] foreign key ([OrderNumber]) references [Order]([OrderNumber]),
	constraint [OrderDetail_Item_FK] foreign key ([ItemIdentity]) references [Item]([Identity])
)
go

create table [Photograph]
(
	[PhotographNumber] uniqueidentifier not null,
	[Name] nvarchar(64) not null,
	[PhotographImage] varbinary(max) not null,
	
	constraint [Photograph_PK] primary key([PhotographNumber])
)
go

create table [ReservedItem]
(
	[ItemIdentity] bigint not null,
	[UserInformationNumber] uniqueidentifier not null,
	[Quantity] int not null,
	[ExpireDateTime] datetime not null,

	constraint [ReservedItem_PK] primary key ([ItemIdentity], [UserInformationNumber])
)
go

create table [OrderNumber]
(
	[OrderNumber] nchar(7) not null,
	
	constraint [OrderNumber_PK] primary key ([OrderNumber])
)
go
