﻿using System;
using System.Data;
using System.Windows.Forms;
using TailIsland.RX;
using TailIsland.PetShop.RetailServiceAgent;

namespace TailIsland.PetShop.CatalogMaintainer
{
    public partial class ProductForm : Form
    {
        public ProductForm()
        {
            InitializeComponent();
        }

        private void ProductForm_Load(object sender, EventArgs e)
        {
            RXForForm.ExecuteEventHandler(
                delegate
                {
                    RXForForm.SetControlMaxLengths(this, retailDataSet.Product);
                });
        }

        private void ProductForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            RXForForm.ExecuteEventHandler(
                delegate
                {
                    try
                    {
                        if (DialogResult != DialogResult.OK)
                        {
                            return;
                        }

                        productBindingSource.EndEdit();
                        if (retailDataSet.GetChanges() == null)
                        {
                            return;
                        }

                        RXForForm.CallValidators(retailDataSet.Product);
                        if (retailDataSet.HasErrors)
                        {
                            MessageBox.Show(this, RXForForm.CollectDataSetErrors(retailDataSet), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
                            e.Cancel = true;
                            return;
                        }

                        CallUpdate();
                        if (retailDataSet.HasErrors)
                        {
                            MessageBox.Show(this, RXForForm.CollectDataSetErrors(retailDataSet), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
                            e.Cancel = true;
                            return;
                        }
                    }
                    catch (Exception)
                    {
                        e.Cancel = true;

                        throw;
                    }
                });
        }

        private void CallUpdate()
        {
            RetailDataSet ds;
 
            using (ProductWebService ws = new ProductWebService())
            {
                RXForForm.InitializeServiceAgent(ws);

                ds = ws.Update(retailDataSet);
            }

            retailDataSet.Clear();
            retailDataSet.Merge(ds);
        }

        private void itemDataGridView_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.ColumnIndex < 0 || e.RowIndex < 0)
            {
                return;
            }

            bindingNavigatorEditItem_Click(sender, e);
        }

        private void bindingNavigatorAddNewItem_Click(object sender, EventArgs e)
        {
            RXForForm.ExecuteEventHandler(
                delegate
                {
                    productBindingSource.EndEdit();

                    using (ItemForm f = new ItemForm())
                    {
                        f.retailDataSet.Merge(retailDataSet);
                        SetInitialValue((RetailDataSet._ItemRow)((DataRowView)f.itemBindingSource.AddNew()).Row);

                        if (f.ShowDialog(this) != DialogResult.OK)
                        {
                            return;
                        }

                        retailDataSet.Merge(f.retailDataSet);
                    }
                });
        }

        private void SetInitialValue(RetailDataSet._ItemRow dataRow)
        {
            dataRow.Identity = retailDataSet._Item.Count == 0 ? 1L : (long)retailDataSet._Item.Compute("Max(Identity)", null) + 1;
            dataRow.ProductIdentity = ((RetailDataSet.ProductRow)((DataRowView)productBindingSource.Current).Row).Identity;
            dataRow.Name = string.Empty;
            dataRow.Price = 0M;
            dataRow.Quantity = 0;
        }

        private void bindingNavigatorEditItem_Click(object sender, EventArgs e)
        {
            RXForForm.ExecuteEventHandler(
                delegate
                {
                    productBindingSource.EndEdit();

                    if (itemBindingSource.Current == null)
                    {
                        return;
                    }

                    using (ItemForm f = new ItemForm())
                    {
                        f.retailDataSet.Merge(retailDataSet);
                        f.itemBindingSource.Position = RXForForm.GetBindingSourcePosition(f.itemBindingSource, ((DataRowView)itemBindingSource.Current).Row);

                        if (f.ShowDialog(this) != DialogResult.OK)
                        {
                            return;
                        }

                        retailDataSet.Merge(f.retailDataSet);
                    }
                });
        }

        private void bindingNavigatorDeleteItem_Click(object sender, EventArgs e)
        {
            RXForForm.ExecuteEventHandler(
                delegate
                {
                    if (MessageBox.Show(this, "削除してもよろしいですか？", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) != DialogResult.Yes)
                    {
                        return;
                    }

                    RXForForm.DeleteDataRow(((DataRowView)itemBindingSource.Current).Row);
                });
        }
    }
}
