﻿using System;
using System.Data;
using System.Windows.Forms;
using TailIsland.RX;
using TailIsland.PetShop.RetailServiceAgent;

namespace TailIsland.PetShop.CatalogMaintainer
{
    public partial class ProductsForm : Form
    {
        public ProductsForm()
        {
            InitializeComponent();
        }

        private void ProductsForm_Load(object sender, EventArgs e)
        {
            RXForForm.ExecuteEventHandler(
                delegate
                {
                    try
                    {
                        using (CategoryWebService ws = new CategoryWebService())
                        {
                            RXForForm.InitializeServiceAgent(ws);

                            retailDataSet.Merge(ws.Fill());
                        }
                    }
                    catch (Exception)
                    {
                        Close();

                        throw;
                    }
                });
        }

        private void fillButton_Click(object sender, EventArgs e)
        {
            RXForForm.ExecuteEventHandler(
                delegate
                {
                    retailDataSet.Product.Clear();
                    retailDataSet._Item.Clear();

                    using (ProductWebService ws = new ProductWebService())
                    {
                        RXForForm.InitializeServiceAgent(ws);

                        retailDataSet.Merge(ws.FillByCategoryIdentityWithChildren((long)categoryComboBox.SelectedValue));
                    }
                });
        }

        private void productDataGridView_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            RXForForm.ExecuteEventHandler(
                delegate
                {
                    if (e.ColumnIndex < 0 || e.RowIndex < 0)
                    {
                        return;
                    }

                    bindingNavigatorEditItem_Click(sender, e);
                });
        }

        private void bindingNavigatorAddNewItem_Click(object sender, EventArgs e)
        {
            RXForForm.ExecuteEventHandler(
                delegate
                {
                    using (ProductForm f = new ProductForm())
                    {
                        f.retailDataSet.Merge(retailDataSet);
                        
                        SetInitialValue((RetailDataSet.ProductRow)((DataRowView)f.productBindingSource.AddNew()).Row);

                        if (f.ShowDialog(this) != DialogResult.OK)
                        {
                            return;
                        }

                        retailDataSet.Merge(f.retailDataSet);
                        retailDataSet.AcceptChanges();
                    }
                });
        }

        private void SetInitialValue(RetailDataSet.ProductRow dataRow)
        {
            dataRow.Identity = retailDataSet.Product.Count == 0 ? 1L : (long)retailDataSet.Product.Compute("Max(Identity)", null) + 1;
            dataRow.CategoryIdentity = (long)categoryComboBox.SelectedValue;
            dataRow.Name = string.Empty;
            dataRow.Description = string.Empty;
            dataRow.PhotographNumber = new Guid();
        }

        private void bindingNavigatorEditItem_Click(object sender, EventArgs e)
        {
            RXForForm.ExecuteEventHandler(
                delegate
                {
                    using (ProductForm f = new ProductForm())
                    {
                        f.retailDataSet.Merge(retailDataSet);
                        f.productBindingSource.Position = RXForForm.GetBindingSourcePosition(f.productBindingSource, ((DataRowView)productBindingSource.Current).Row);

                        if (f.ShowDialog(this) != DialogResult.OK)
                        {
                            return;
                        }

                        retailDataSet.Merge(f.retailDataSet);
                        retailDataSet.AcceptChanges();
                    }
                });
        }

        private void bindingNavigatorDeleteItem_Click(object sender, EventArgs e)
        {
            RXForForm.ExecuteEventHandler(
                delegate
                {
                    if (MessageBox.Show(this, "削除してもよろしいですか？", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) != DialogResult.Yes)
                    {
                        return;
                    }

                    RXForForm.DeleteDataRow(((DataRowView)productBindingSource.Current).Row);

                    CallUpdate();
                    if (retailDataSet.HasErrors)
                    {
                        MessageBox.Show(this, RXForForm.CollectDataSetErrors(retailDataSet), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
                        return;
                    }

                    retailDataSet.AcceptChanges();
                });
        }

        private void CallUpdate()
        {
            RetailDataSet ds;

            using (ProductWebService ws = new ProductWebService())
            {
                RXForForm.InitializeServiceAgent(ws);

                ds = ws.Update(retailDataSet);
            }

            retailDataSet.Clear();
            retailDataSet.Merge(ds);
        }
    }
}
