﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using TailIsland.RX;

namespace TailIsland.PetShop.Retail.Test
{
    /// <summary>
    ///    Itemに対するユニット・テストです。
    /// </summary>
    [TestClass]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1001")]
    public class ItemUnitTest
    {
        /// <summary>
        ///    テストに使用する型付DataSet。
        /// </summary>
        private RetailDataSet dataSet;

        /// <summary>
        ///    テストに使用する型付DataTable。
        /// </summary>
        private RetailDataSet.ItemDataTable dataTable;

        /// <summary>
        ///    環境を初期化します。
        /// </summary>
        [TestInitialize]
        public void TestInitialize()
        {
            //  フィールドを初期化します。
            dataSet = new RetailDataSet();
            dataTable = dataSet.Item;

            //  データベースを初期化します。
            DataBaseMaintainer.Initialize();
        }

        /// <summary>
        ///    環境をクリーンアップします。
        /// </summary>
        [TestCleanup]
        public void TestCleanup()
        {
            //  フィールドをクリーンアップします。
            dataSet.Dispose();

            //  データベースをクリーンアップします。
            DataBaseMaintainer.Cleanup();
        }

        /// <summary>
        ///    入力値検証をテストします。
        /// </summary>
        [TestMethod]
        public void Validate()
        {
            //  追加よりも変更の方が楽なので、まずはデータを取得します。
            Item.Fill(dataSet);

            //  不正な入力値を作成。それぞれが独立であることが分かっているので、組み合わせのテストはしません。
            dataTable[0].Name = string.Empty;
            dataTable[0].Price = -1M;
            dataTable[1].Price = 1.1M;       //  日本なので、価格は整数でなければなりません。
            dataTable[0].Quantity = -1;

            //  在庫数チェックは、OrderUnitTestのMakeOrder()メソッドで実施します。

            //  Update()メソッドを呼びます。Validatorが呼び出され、入力値検証が実行されます。
            Product.Update(dataSet);

            //  正しくエラーが設定されたか検証します。
            Assert.AreEqual("名称は必須入力です。", dataTable[0].GetColumnError(dataTable.NameColumn));
            Assert.AreEqual("価格は正の数でなければなりません。", dataTable[0].GetColumnError(dataTable.PriceColumn));
            Assert.AreEqual("価格は整数でなければなりません。", dataTable[1].GetColumnError(dataTable.PriceColumn));
            Assert.AreEqual("在庫数量は正の数でなければなりません。", dataTable[0].GetColumnError(dataTable.QuantityColumn));
        }

        /// <summary>
        ///    Fill()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void Fill()
        {
            //  Fill()メソッドを呼び出せることを確認します。
            Item.Fill(dataSet);
        }

        /// <summary>
        ///    FillByProductIdentity()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void FillByProductIdentity()
        {
            //  FillByProductIdentity()メソッドを呼び出せることを確認します。
            Item.FillByProductIdentity(dataSet, 1);
        }

        /// <summary>
        ///    FillByIdentity()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void FillByItemIdentity()
        {
            //  FillByItemIdentity()メソッドを呼び出せることを確認します。
            Item.FillByItemIdentity(dataSet, 1);
        }

        /// <summary>
        ///    FillWithChildren()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void FillWithChildren()
        {
            //  FillWithChildren()メソッドを呼び出せることを確認します。
            Item.FillWithChildren(dataSet);
        }

        /// <summary>
        ///    GetData()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void GetData()
        {
            //  GetData()メソッドを呼び出せることを確認します。
            Item.GetData();
        }
    }
}
