﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using TailIsland.RX;

namespace TailIsland.PetShop.Retail.Test
{
    /// <summary>
    ///    Photographに対するユニット・テストです。
    /// </summary>
    [TestClass]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1001")]
    public class PhotographUnitTest
    {
        /// <summary>
        ///    テストに使用する型付DataSet。
        /// </summary>
        private RetailDataSet dataSet;

        /// <summary>
        ///    テストに使用する型付DataTable。
        /// </summary>
        private RetailDataSet.PhotographDataTable dataTable;

        /// <summary>
        ///    環境を初期化します。
        /// </summary>
        [TestInitialize]
        public void TestInitialize()
        {
            //  フィールドを初期化します。
            dataSet = new RetailDataSet();
            dataTable = dataSet.Photograph;

            //  データベースを初期化します。
            DataBaseMaintainer.Initialize();
        }

        /// <summary>
        ///    環境をクリーンアップします。
        /// </summary>
        [TestCleanup]
        public void TestCleanup()
        {
            //  フィールドをクリーンアップします。
            dataSet.Dispose();

            //  データベースをクリーンアップします。
            DataBaseMaintainer.Cleanup();
        }

        /// <summary>
        ///    入力値検証をテストします。
        /// </summary>
        [TestMethod]
        public void Validate()
        {
            //  追加よりも変更の方が楽なので、まずはデータを取得します。
            Photograph.Fill(dataSet);

            //  不正な入力値を作成。それぞれが独立であることが分かっているので、組み合わせのテストはしません。
            dataTable[0].Name = string.Empty;
            dataTable[0].PhotographImage = new byte[] { };

            //  Update()メソッドを呼びます。Validatorが呼び出され、入力値検証が実行されます。
            Photograph.Update(dataSet);

            //  正しくエラーが設定されたか検証します。
            Assert.AreEqual("名称は必須入力です。", dataTable[0].GetColumnError(dataTable.NameColumn));
            Assert.AreEqual("写真イメージは必須入力です。", dataTable[0].GetColumnError(dataTable.PhotographImageColumn));
        }

        /// <summary>
        ///    Fill()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void Fill()
        {
            //  Fill()メソッドを呼び出せることを確認します。
            Photograph.Fill(dataSet);
        }

        /// <summary>
        ///    FillByPhotographNumber()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void FillByPhotographNumber()
        {
            //  FillByPhotographNumber()メソッドを呼び出せることを確認します。
            Photograph.FillByPhotographNumber(dataSet, new Guid("D288ECDE-B103-4b9c-9EBD-DDBEC99F9836"));
        }

        /// <summary>
        ///    FillWithChildren()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void FillWithChildren()
        {
            //  FillWithChildren()メソッドを呼び出せることを確認します。
            Photograph.FillWithChildren(dataSet);
        }

        /// <summary>
        ///    GetData()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void GetData()
        {
            //  GetData()メソッドを呼び出せることを確認します。
            Photograph.GetData();
        }
    }
}
