﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using TailIsland.RX;

namespace TailIsland.PetShop.Retail.Test
{
    /// <summary>
    ///    Productに対するユニット・テストです。
    /// </summary>
    [TestClass]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1001")]
    public class ProductUnitTest
    {
        /// <summary>
        ///    テストに使用する型付DataSet。
        /// </summary>
        private RetailDataSet dataSet;

        /// <summary>
        ///    テストに使用する型付DataTable。
        /// </summary>
        private RetailDataSet.ProductDataTable dataTable;

        /// <summary>
        ///    環境を初期化します。
        /// </summary>
        [TestInitialize]
        public void TestInitialize()
        {
            //  フィールドを初期化します。
            dataSet = new RetailDataSet();
            dataTable = dataSet.Product;

            //  データベースを初期化します。
            DataBaseMaintainer.Initialize();
        }

        /// <summary>
        ///    環境をクリーンアップします。
        /// </summary>
        [TestCleanup]
        public void TestCleanup()
        {
            //  フィールドをクリーンアップします。
            dataSet.Dispose();

            //  データベースをクリーンアップします。
            DataBaseMaintainer.Cleanup();
        }

        /// <summary>
        ///    入力値検証をテストします。
        /// </summary>
        [TestMethod]
        public void Validate()
        {
            //  追加よりも変更の方が楽なので、まずはデータを取得します。
            Product.Fill(dataSet);

            //  不正な入力値を作成。それぞれが独立であることが分かっているので、組み合わせのテストはしません。
            dataTable[0].Name = string.Empty;
            dataTable[0].Description = string.Empty;
            foreach (RetailDataSet.ItemRow dr in dataTable[0].GetItemRows())
            {
                dr.Delete();
            }

            //  Update()メソッドを呼びます。Validatorが呼び出され、入力値検証が実行されます。
            Product.Update(dataSet);

            //  正しくエラーが設定されたか検証します。
            Assert.AreEqual("名称は必須入力です。", dataTable[0].GetColumnError(dataTable.NameColumn));
            Assert.AreEqual("説明文は必須入力です。", dataTable[0].GetColumnError(dataTable.DescriptionColumn));
            Assert.AreEqual("商品には最低一つの品目が必要です。", dataTable[0].RowError);
        }

        /// <summary>
        ///    Fill()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void Fill()
        {
            //  Fill()メソッドを呼び出せることを確認します。
            Product.Fill(dataSet);
        }

        /// <summary>
        ///    FillByCategoryIdentity()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void FillByCategoryIdentity()
        {
            //  FillByCategoryIdentity()メソッドを呼び出せることを確認します。
            Product.FillByCategoryIdentity(dataSet, 1);
        }

        /// <summary>
        ///    FillWithChildren()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void FillWithChildren()
        {
            //  FillWithChildren()メソッドを呼び出せることを確認します。
            Product.FillWithChildren(dataSet);
        }

        /// <summary>
        ///    FillByCategoryIdentityWithChildren()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void FillByCategoryIdentityWithChildren()
        {
            //  FillByCategoryIdentityWithChildren()メソッドを呼び出せることを確認します。
            Product.FillByCategoryIdentityWithChildren(dataSet, 1);
        }

        /// <summary>
        ///    GetData()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void GetData()
        {
            //  GetData()メソッドを呼び出せることを確認します。
            Product.GetData();
        }
    }
}
