﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using TailIsland.RX;

namespace TailIsland.PetShop.Retail.Test
{
    /// <summary>
    ///    UserInformationに対するユニット・テストです。
    /// </summary>
    [TestClass]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1001")]
    public class UserInformationUnitTest
    {
        /// <summary>
        ///    テストに使用する型付DataSet。
        /// </summary>
        private RetailDataSet dataSet;

        /// <summary>
        ///    テストに使用する型付DataTable。
        /// </summary>
        private RetailDataSet.UserInformationDataTable dataTable;

        /// <summary>
        ///    環境を初期化します。
        /// </summary>
        [TestInitialize]
        public void TestInitialize()
        {
            //  フィールドを初期化します。
            dataSet = new RetailDataSet();
            dataTable = dataSet.UserInformation;

            //  データベースを初期化します。
            DataBaseMaintainer.Initialize();
        }

        /// <summary>
        ///    環境をクリーンアップします。
        /// </summary>
        [TestCleanup]
        public void TestCleanup()
        {
            //  フィールドをクリーンアップします。
            dataSet.Dispose();

            //  データベースをクリーンアップします。
            DataBaseMaintainer.Cleanup();
        }

        /// <summary>
        ///    入力値検証をテストします。
        /// </summary>
        [TestMethod]
        public void Validate()
        {
            //  追加よりも変更の方が楽なので、まずはデータを取得します。
            UserInformation.Fill(dataSet);

            //  不正な入力値を作成。それぞれが独立であることが分かっているので、組み合わせのテストはしません。
            dataTable[0].ZipCode = string.Empty;
            dataTable[1].ZipCode = "x000-0000x";
            dataTable[0].Address1 = string.Empty;
            dataTable[0].PhoneNumber = string.Empty;
            dataTable[1].PhoneNumber = "x00-0000-0000x";

            //  Update()メソッドを呼びます。Validatorが呼び出され、入力値検証が実行されます。
            UserInformation.Update(dataSet);

            //  正しくエラーが設定されたか検証します。
            Assert.AreEqual("郵便番号は必須入力です。", dataTable[0].GetColumnError(dataTable.ZipCodeColumn));
            Assert.AreEqual("郵便番号は半角数字3桁と「-（半角ハイフン）」、半角数字4桁で入力してください。", dataTable[1].GetColumnError(dataTable.ZipCodeColumn));
            Assert.AreEqual("住所1は必須入力です。", dataTable[0].GetColumnError(dataTable.Address1Column));
            Assert.AreEqual("電話番号は必須入力です。", dataTable[0].GetColumnError(dataTable.PhoneNumberColumn));
            Assert.AreEqual("電話番号は半角数字および「-（半角ハイフン）」で入力してください。", dataTable[1].GetColumnError(dataTable.PhoneNumberColumn));
        }

        /// <summary>
        ///    Fill()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void Fill()
        {
            //  Fill()メソッドを呼び出せることを確認します。
            UserInformation.Fill(dataSet);
        }

        /// <summary>
        ///    FillByUserInformationNumber()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void FillByUserInformationNumber()
        {
            //  FillByUserInformationNumber()メソッドを呼び出せることを確認します。
            UserInformation.FillByUserInformationNumber(dataSet, new Guid("25b60925-a865-4ca8-90fe-6c6ed493e851"));
        }

        /// <summary>
        ///    FillWithChildren()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void FillWithChildren()
        {
            //  FillWithChildren()メソッドを呼び出せることを確認します。
            UserInformation.FillWithChildren(dataSet);
        }

        /// <summary>
        ///    GetData()メソッドをテストします。
        /// </summary>
        [TestMethod]
        public void GetData()
        {
            //  GetData()メソッドを呼び出せることを確認します。
            UserInformation.GetData();
        }
    }
}
