﻿using System;
using TailIsland.RX;

namespace TailIsland.PetShop.Retail
{
    /// <summary>
    ///    注文明細に対する入力値検証を担当するクラスです。
    /// </summary>
    internal static class OrderDetailValidator
    {
        /// <summary>
        ///    入力値を検証します。
        /// </summary>
        /// <param name="dataTable">入力値が入った型付DataTable。</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811")]
        internal static void Validate(RetailDataSet.OrderDetailDataTable dataTable)
        {
            //  すべての追加/変更された行に対して……
            foreach (RetailDataSet.OrderDetailRow dr in RXForValidator.GetAddedOrModifiedRows(dataTable))
            {
                //  価格が正の数になっていることをチェック。
                RXForValidator.AddColumnError(dr, dataTable.PriceColumn, "価格は正の数でなければなりません。",
                    delegate
                    {
                        return dr.Price < 0;
                    });

                //  価格が整数になっていることをチェック。
                RXForValidator.AddColumnError(dr, dataTable.PriceColumn, "価格は整数でなければなりません。",
                    delegate
                    {
                        return dr.Price != Math.Round(dr.Price);
                    });

                //  購入数量が0ではない正の数になっていることをチェック。
                RXForValidator.AddColumnError(dr, dataTable.QuantityColumn, "購入数量は正の数でなければなりません。",
                    delegate
                    {
                        return dr.Quantity <= 0;
                    });

                //  在庫が残っていることをチェック。
                RXForValidator.AddColumnError(dr, dataTable.QuantityColumn, string.Format("{0}（{1}）の在庫がありませんでした。", dr.ItemRow.ProductRow.Name, dr.ItemRow.Name),
                    delegate
                    {
                        //  MakeOrder()メソッドの中ですでに在庫割り当て済みなので、ここでは購入数量は使用しません。
                        return Item.CalculateNotReservedQuantity(dr.ItemIdentity, dr.OrderRow.UserInformationNumber) < 0;
                    });
            }
        }
    }
}
