﻿using System;
using System.Text.RegularExpressions;
using TailIsland.RX;

namespace TailIsland.PetShop.Retail
{
    /// <summary>
    ///    注文に対する入力値検証を担当するクラスです。
    /// </summary>
    internal static class OrderValidator
    {
        /// <summary>
        ///    入力値を検証します。
        /// </summary>
        /// <param name="dataTable">入力値が入った型付DataTable。</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811")]
        internal static void Validate(RetailDataSet.OrderDataTable dataTable)
        {
            //  すべての追加/変更された行に対して……
            foreach (RetailDataSet.OrderRow dr in RXForValidator.GetAddedOrModifiedRows(dataTable))
            {
                //  必須チェック。
                RXForValidator.AddColumnErrorIfRequireIsNot(dr, dataTable.ShippingZipCodeColumn);
                RXForValidator.AddColumnErrorIfRequireIsNot(dr, dataTable.ShippingAddress1Column);
                RXForValidator.AddColumnErrorIfRequireIsNot(dr, dataTable.ShippingPhoneNumberColumn);
                RXForValidator.AddColumnErrorIfRequireIsNot(dr, dataTable.ShippingNameColumn);

                //  郵便番号の形式をチェック。
                RXForValidator.AddColumnError(dr, dataTable.ShippingZipCodeColumn, "出荷先郵便番号は半角数字3桁と「-（半角ハイフン）」、半角数字4桁で入力してください。",
                    delegate
                    {
                        return !string.IsNullOrEmpty(dr.ShippingZipCode) && !Regex.IsMatch(dr.ShippingZipCode, @"^[0-9]{3}-[0-9]{4}$");
                    });

                //  電話番号の形式をチェック。
                RXForValidator.AddColumnError(dr, dataTable.ShippingPhoneNumberColumn, "出荷先電話番号は半角数字および「-（半角ハイフン）」で入力してください。",
                    delegate
                    {
                        return !string.IsNullOrEmpty(dr.ShippingPhoneNumber) && !Regex.IsMatch(dr.ShippingPhoneNumber, @"^[0-9\-]+$");
                    });

                //  クレジット・カード承認の成功をチェック。
                RXForValidator.AddColumnError(dr, dataTable.CreditCardChargeNumberColumn, "クレジット・カードを承認できませんでした。クレジット・カード情報を確認してください。",
                    delegate
                    {
                        return string.IsNullOrEmpty(dr.CreditCardChargeNumber);
                    });

                //  注文明細の存在をチェック。
                RXForValidator.AddRowError(dr, "注文には最低一つの注文明細が必要です。",
                    delegate
                    {
                        return dr.GetOrderDetailRows().Length == 0;
                    });
            }
        }
    }
}
