﻿using System;
using System.Data;
using TailIsland.RX;

namespace TailIsland.PetShop.Retail
{
    /// <summary>
    ///    予約済品目に対する入力値検証を担当するクラスです。
    /// </summary>
    internal static class ReservedItemValidator
    {
        /// <summary>
        ///    入力値を検証します。
        /// </summary>
        /// <param name="dataTable">入力値が入った型付DataTable。</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1801")]
        internal static void Validate(RetailDataSet.ReservedItemDataTable dataTable)
        {
            //  すべての追加/変更された行に対して……
            foreach (RetailDataSet.ReservedItemRow dr in RXForValidator.GetAddedOrModifiedRows(dataTable))
            {
                //  予約数量が0を含まない正の数になっていることをチェック。
                RXForValidator.AddColumnError(dr, dataTable.QuantityColumn, "予約数量は正の数でなければなりません。",
                    delegate
                    {
                        return dr.Quantity <= 0;
                    });

                //  有効期限が未来になっていることをチェック。
                RXForValidator.AddColumnError(dr, dataTable.ExpireDateTimeColumn, "有効期限には未来の日付が設定されていなければなりません。",
                    delegate
                    {
                        return dr.ExpireDateTime < DateTime.Now;
                    });
            }
        }
    }
}
