﻿using System;
using System.Text.RegularExpressions;
using TailIsland.RX;

namespace TailIsland.PetShop.Retail
{
    /// <summary>
    ///    UserInformationに対する入力値検証を担当するクラスです。
    /// </summary>
    internal static class UserInformationValidator
    {
        /// <summary>
        ///    入力値を検証します。
        /// </summary>
        /// <param name="dataTable">入力値が入った型付DataTable。</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1801")]
        internal static void Validate(RetailDataSet.UserInformationDataTable dataTable)
        {
            //  すべての追加/変更された行に対して……
            foreach (RetailDataSet.UserInformationRow dr in RXForValidator.GetAddedOrModifiedRows(dataTable))
            {
                //  必須チェック。
                RXForValidator.AddColumnErrorIfRequireIsNot(dr, dataTable.ZipCodeColumn);
                RXForValidator.AddColumnErrorIfRequireIsNot(dr, dataTable.Address1Column);
                RXForValidator.AddColumnErrorIfRequireIsNot(dr, dataTable.PhoneNumberColumn);
                RXForValidator.AddColumnErrorIfRequireIsNot(dr, dataTable.NameColumn);

                //  郵便番号の形式をチェック。
                RXForValidator.AddColumnError(dr, dataTable.ZipCodeColumn, "郵便番号は半角数字3桁と「-（半角ハイフン）」、半角数字4桁で入力してください。",
                    delegate
                    {
                        return !string.IsNullOrEmpty(dr.ZipCode) && !Regex.IsMatch(dr.ZipCode, @"^[0-9]{3}-[0-9]{4}$");
                    });

                //  電話番号の形式をチェック。
                RXForValidator.AddColumnError(dr, dataTable.PhoneNumberColumn, "電話番号は半角数字および「-（半角ハイフン）」で入力してください。",
                    delegate
                    {
                        return !string.IsNullOrEmpty(dr.PhoneNumber) && !Regex.IsMatch(dr.PhoneNumber, @"^[0-9\-]+$");
                    });
            }
        }
    }
}
