﻿using System;
using System.IO;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using TailIsland.RX;
using TailIsland.PetShop.Retail;

namespace TailIsland.PetShop.StoreFront.Catalog
{
    /// <summary>
    ///    写真を取得するためのWebFormです。CGIのように使用します。
    /// </summary>
    public partial class PhotographWebForm : Page
    {
        /// <summary>
        ///    ページがロードされた時のイベント・ハンドラです。
        /// </summary>
        /// <param name="sender">イベントの送信元。</param>
        /// <param name="e">イベント。</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            RXForWebForm.ExecuteEventHandler(
                delegate
                {
                    //  写真を取得します。
                    RetailDataSet ds = new RetailDataSet();
                    Photograph.FillByPhotographNumber(ds, new Guid(Request["photographNumber"]));

                    //  レスポンスのコンテンツ・タイプをimage/jpgに設定します。
                    Response.ContentType = "image/jpg";

                    //  写真を取得できた場合は……。
                    if (ds.Photograph.Count > 0)
                    {
                        //  写真のイメージをバイナリで出力します。
                        Response.BinaryWrite(ds.Photograph[0].PhotographImage);
                    }
                    else
                    {
                        //  NowPrintingのイメージをバイナリで出力します。
                        Response.BinaryWrite(GetNowPrintingImage());
                    }

                    //  出力を終了します。
                    Response.End();
                });
        }

        /// <summary>
        ///    NowPrintingのイメージを取得します。
        /// </summary>
        /// <returns>NowPrintingのイメージのバイナリ。</returns>
        private byte[] GetNowPrintingImage()
        {
            //  NowPrinting.jpgファイルを開きます。
            using (Stream s = new BufferedStream(new FileStream(MapPath("./NowPrinting.jpg"), FileMode.Open, FileAccess.Read)))
            {
                //  戻り値を宣言します。
                byte[] bs = new byte[s.Length];

                //  ファイルの中身を取得します。
                s.Read(bs, 0, (int)s.Length);

                //  取得した中身をリターンします。
                return bs;
            }
        }
    }
}
