﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using TailIsland.RX;
using TailIsland.PetShop.Retail;

namespace TailIsland.PetShop.StoreFront.Purchase
{
    /// <summary>
    ///    注文詳細のためのWebFormです。ショッピング・カートになります。
    /// </summary>
    public partial class OrderDetailsWebForm : Page
    {
        /// <summary>
        ///    RetailDataSetを取得/設定します。データはセッションを利用して保持します。
        /// </summary>
        protected RetailDataSet RetailDataSet
        {
            get { return (RetailDataSet)Session["TailIsland.PetShop.StoreFront.Purchase.RetailDataSet"]; }
            set { Session["TailIsland.PetShop.StoreFront.Purchase.RetailDataSet"] = value; }
        }

        /// <summary>
        ///    合計額を計算します。
        /// </summary>
        protected decimal TotalPrice
        {
            get
            {
                //  合計額を計算してリターン。
                return Order.GetTotalPrice(RetailDataSet, RetailDataSet.Order[0].OrderNumber);
            }
        }

        /// <summary>
        ///    ページがロードされた時のイベント・ハンドラです。
        /// </summary>
        /// <param name="sender">イベントの送信元。</param>
        /// <param name="e">イベント。</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            RXForWebForm.ExecuteEventHandler(
                delegate
                {
                    //  ポスト・バックでなければ……。
                    if (!IsPostBack)
                    {
                        //  RetailDataSetがnullの場合は……。
                        if (RetailDataSet == null)
                        {
                            //  RetailDataSetを生成します。
                            RetailDataSet = new RetailDataSet();

                            //  利用者情報を取得します。
                            FillUserInformation();

                            //  注文を追加します。
                            AddOrder();
                        }

                        //  品目を取得します。
                        FillItem();

                        //  注文明細を追加するか、数量を増やします。
                        AddOrIncrementOrderDetail();

                        //  データ・バインディングを実行します。
                        DataBind();
                    }
                });
        }

        /// <summary>
        ///    注文明細のRepeaterでコマンドが発行された時のイベント・ハンドラです。
        /// </summary>
        /// <param name="source">イベントの送信元。</param>
        /// <param name="e">イベント。</param>
        protected void orderDetailRepeater_ItemCommand(object source, RepeaterCommandEventArgs e)
        {
            RXForWebForm.ExecuteEventHandler(
                delegate
                {
                    //  コマンド名が……。
                    switch (e.CommandName)
                    {
                        //  更新の時は……。
                        case "Update":
                            //  注文明細を更新します。
                            UpdateOrderDetail(RetailDataSet.OrderDetail.FindByIdentity(Convert.ToInt64(e.CommandArgument)), Convert.ToInt32(((TextBox)e.Item.FindControl("orderDetailQuantityTextBox")).Text));

                            //  データ・バインディングを実行します。
                            DataBind();
                            
                            //  ブレイク。
                            break;

                        //  削除の時は……。
                        case "Delete":
                            //  数量がゼロになるように、注文明細を更新します。
                            UpdateOrderDetail(RetailDataSet.OrderDetail.FindByIdentity(Convert.ToInt64(e.CommandArgument)), 0);

                            //  データ・バインディングを実行します。
                            DataBind();

                            //  ブレイク。
                            break;
                    }
                });
        }

        /// <summary>
        ///    [レジに進む]ボタンがクリックされたときのイベント・ハンドラです。
        /// </summary>
        /// <param name="sender">イベントの送信元。</param>
        /// <param name="e">イベント。</param>
        protected void goToRegister_Click(object sender, EventArgs e)
        {
            RXForWebForm.ExecuteEventHandler(
                delegate
                {
                    //  OrderWebFormにリダイレクトします。
                    Response.Redirect("../Purchase/OrderWebForm.aspx");
                });
        }

        /// <summary>
        ///    利用者情報を取得します。
        /// </summary>
        private void FillUserInformation()
        {
            //  利用者情報を取得します。
            RetailDataSet ds = new RetailDataSet();
            UserInformation.FillByUserInformationNumber(ds, (Guid)Membership.GetUser(HttpContext.Current.User.Identity.Name).ProviderUserKey);

            //  マージ。
            RetailDataSet.Merge(ds);
        }

        /// <summary>
        ///    注文を追加します。
        /// </summary>
        private void AddOrder()
        {
            //  ダミーの注文を追加します。
            RetailDataSet.Order.AddOrderRow(
                "x",
                RetailDataSet.UserInformation[0],
                RetailDataSet.UserInformation[0].ZipCode,
                RetailDataSet.UserInformation[0].Address1,
                RetailDataSet.UserInformation[0].Address2,
                RetailDataSet.UserInformation[0].PhoneNumber,
                RetailDataSet.UserInformation[0].Name,
                "x",
                "x",
                "x",
                0,
                0);
        }

        /// <summary>
        ///    品目を取得します。
        /// </summary>
        private void FillItem()
        {
            //  品目を取得します。
            RetailDataSet ds = new RetailDataSet();
            Item.FillByItemIdentity(ds, Convert.ToInt64(Request["itemIdentity"]));

            //  マージ。
            RetailDataSet.Merge(ds);
        }

        /// <summary>
        ///    注文明細を追加するか、数量を増やします。
        /// </summary>
        private void AddOrIncrementOrderDetail()
        {
            //  品目の識別子で、すでに取得済みのデータをメモリ内で検索します。
            RetailDataSet.OrderDetailRow[] dr = (RetailDataSet.OrderDetailRow[])RetailDataSet.OrderDetail.Select(string.Format("ItemIdentity = {0}", Convert.ToInt64(Request["itemIdentity"])));

            //  品目を取得していない場合は……。
            if (dr.Length == 0)
            {
                //  注文明細を追加します。
                AddOrderDetail(Convert.ToInt64(Request["itemIdentity"]));
            }
            else
            {
                //  数量が1増えるように、注文明細を更新します。
                UpdateOrderDetail(dr[0], dr[0].Quantity + 1);
            }
        }

        /// <summary>
        ///    注文明細を追加します。
        /// </summary>
        /// <param name="itemIdentity">品目の識別子。</param>
        private void AddOrderDetail(long itemIdentity)
        {
            //  注文明細を追加します。
            RetailDataSet.OrderDetail.AddOrderDetailRow(
                RetailDataSet.OrderDetail.Count == 0 ? 1L : (long)RetailDataSet.OrderDetail.Compute("Max(Identity)", null) + 1,
                RetailDataSet.Order[0],
                RetailDataSet.Item.FindByIdentity(itemIdentity),
                RetailDataSet.Item.FindByIdentity(itemIdentity).Price,
                1);

            //  予約し、リフレッシュします。
            ResetveAndRefresh();
        }

        /// <summary>
        ///    注文明細を更新します。
        /// </summary>
        /// <param name="dataRow"></param>
        /// <param name="quantity"></param>
        private void UpdateOrderDetail(RetailDataSet.OrderDetailRow dataRow, int quantity)
        {
            //  数量を変更し、価格を再計算します。
            dataRow.Quantity = quantity;
            dataRow.Price = dataRow.ItemRow.Price * dataRow.Quantity;

            //  予約し、リフレッシュします。
            ResetveAndRefresh();
        }

        /// <summary>
        ///    予約し、リフレッシュします。
        /// </summary>
        private void ResetveAndRefresh()
        {
            //  予約します。
            Reserve();

            //  リフレッシュします。
            Refresh();
        }

        /// <summary>
        ///    予約します。
        /// </summary>
        private void Reserve()
        {
            //  エラーをクリアします。
            RXForBusiness.ClearDataSetErrors(RetailDataSet);

            //  予約します。
            Order.Reserve(RetailDataSet, RetailDataSet.Order[0].OrderNumber);

            //  もしエラーがあれば、エラー情報を画面に表示します。
            errorLabel.Text = HttpUtility.HtmlEncode(RXForWebForm.CollectDataSetErrors(RetailDataSet)).Replace(Environment.NewLine, "<br />");
        }

        /// <summary>
        ///    リフレッシュします。
        /// </summary>
        private void Refresh()
        {
            List<RetailDataSet.OrderDetailRow> l = new List<RetailDataSet.OrderDetailRow>();

            //  数量が0の注文明細のリストを作成します。
            foreach (RetailDataSet.OrderDetailRow dr in RetailDataSet.OrderDetail)
            {
                if (dr.Quantity == 0)
                {
                    l.Add(dr);
                }
            }

            //  数量が0の注文明細をすべて削除します。
            foreach (RetailDataSet.OrderDetailRow dr in l)
            {
                dr.Delete();
            }
        }
    }
}
