﻿using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using TailIsland.RX;
using TailIsland.PetShop.Retail;

namespace TailIsland.PetShop.StoreFront.Purchase
{
    /// <summary>
    ///    注文のためのWebFormです。
    /// </summary>
    public partial class OrderWebForm : Page
    {
        /// <summary>
        ///    RetailDataSetを取得/設定します。データはセッションを利用して保持します。
        /// </summary>
        protected RetailDataSet RetailDataSet
        {
            get { return (RetailDataSet)Session["TailIsland.PetShop.StoreFront.Purchase.RetailDataSet"]; }
            set { Session["TailIsland.PetShop.StoreFront.Purchase.RetailDataSet"] = value; }
        }

        /// <summary>
        ///    ページがロードされた時のイベント・ハンドラです。
        /// </summary>
        /// <param name="sender">イベントの送信元。</param>
        /// <param name="e">イベント。</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            RXForWebForm.ExecuteEventHandler(
                delegate
                {
                    //  ポスト・バックでなければ……。
                    if (!IsPostBack)
                    {
                        //  クレジット・カード有効期限用のドロップ・ダウン・リスト・ボックスを準備します。
                        for (int i = DateTime.Now.Year; i < DateTime.Now.Year + 20; ++i)
                        {
                            creditCardExpireYearDropDownList.Items.Add(string.Format("{0}", i));
                        }

                        //  データ・バインディングを実行します。
                        DataBind();
                    }
                });
        }

        /// <summary>
        ///    [購入を確定する]ボタンがクリックされた時のイベント・ハンドラです。
        /// </summary>
        /// <param name="sender">イベントの送信元。</param>
        /// <param name="e">イベント。</param>
        protected void purchaseButton_Click(object sender, EventArgs e)
        {
            RXForWebForm.ExecuteEventHandler(
                delegate
                {
                    //  入力データを型付DataSetに反映させます。
                    RetailDataSet.Order[0].ShippingZipCode = shippingZipCodeTextBox.Text;
                    RetailDataSet.Order[0].ShippingAddress1 = shippingAddress1TextBox.Text;
                    RetailDataSet.Order[0].ShippingAddress2 = shippingAddress2TextBox.Text;
                    RetailDataSet.Order[0].ShippingPhoneNumber = shippingPhoneNumberTextBox.Text;
                    RetailDataSet.Order[0].ShippingName = shippingNameTextBox.Text;
                    RetailDataSet.Order[0].CreditCardCompany = creditCardCompanyDropDownList.SelectedValue;
                    RetailDataSet.Order[0].CreditCardNumber = creditCardNumberTextBox.Text;
                    RetailDataSet.Order[0].CreditCardExpireMonth = Convert.ToInt32(creditCardExpireMonthDropDownList.SelectedValue);
                    RetailDataSet.Order[0].CreditCardExpireYear = Convert.ToInt32(creditCardExpireYearDropDownList.SelectedValue);

                    //  エラーをクリアします。
                    RXForBusiness.ClearDataSetErrors(RetailDataSet);

                    //  注文します。
                    Order.MakeOrder(RetailDataSet);

                    //  （もしエラーがあるならば）エラーを表示します。
                    errorLabel.Text = HttpUtility.HtmlEncode(RXForWebForm.CollectDataSetErrors(RetailDataSet)).Replace(Environment.NewLine, "<br />");

                    //  エラーがある場合はリターンします。
                    if (RetailDataSet.HasErrors)
                    {
                        return;
                    }

                    //  予約を破棄します。
                    ReservedItem.Clear(RetailDataSet.UserInformation[0].UserInformationNumber);

                    //  注文結果にリダイレクトします。
                    Response.Redirect("../Purchase/OrderResultWebForm.aspx");
                });
        }
    }
}
