﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using TailIsland.RX;

namespace TailIsland.RX.Test
{
    [TestClass]
    public class RXForValidatorUnitTest
    {
        [TestMethod]
        public void AddRowError()
        {
            using (RXDataSet ds = new RXDataSet())
            {
                RXDataSet.T11Row t11r = ds.T11.AddT11Row(1L, "X");

                RXForValidator.AddRowError(t11r, "aaa");
                Assert.AreEqual("aaa", t11r.RowError);

                RXForValidator.AddRowError(t11r, "bbb");
                Assert.AreEqual("aaa\r\nbbb", t11r.RowError);

                RXForValidator.AddRowError(t11r, "ccc");
                Assert.AreEqual("aaa\r\nbbb\r\nccc", t11r.RowError);
            }

            using (RXDataSet ds = new RXDataSet())
            {
                RXDataSet.T11Row t11r = ds.T11.AddT11Row(1L, "X");

                RXForValidator.AddRowError(t11r, "aaa",
                    delegate
                    {
                        return false;
                    });
                Assert.AreEqual(string.Empty, t11r.RowError);

                RXForValidator.AddRowError(t11r, "aaa",
                    delegate
                    {
                        return true;
                    });
                Assert.AreEqual("aaa", t11r.RowError);
            }
        }

        [TestMethod]
        public void AddColumnError()
        {
            using (RXDataSet ds = new RXDataSet())
            {
                RXDataSet.T11Row t11r = ds.T11.AddT11Row(1L, "X");

                RXForValidator.AddColumnError(t11r, ds.T11.ValueColumn, "aaa");
                Assert.AreEqual("aaa", t11r.GetColumnError(ds.T11.ValueColumn));

                RXForValidator.AddColumnError(t11r, ds.T11.ValueColumn, "bbb");
                Assert.AreEqual("aaa\r\nbbb", t11r.GetColumnError(ds.T11.ValueColumn));

                RXForValidator.AddColumnError(t11r, ds.T11.ValueColumn, "ccc");
                Assert.AreEqual("aaa\r\nbbb\r\nccc", t11r.GetColumnError(ds.T11.ValueColumn));
            }

            using (RXDataSet ds = new RXDataSet())
            {
                RXDataSet.T11Row t11r = ds.T11.AddT11Row(1L, "X");

                RXForValidator.AddColumnError(t11r, ds.T11.ValueColumn, "aaa",
                    delegate
                    {
                        return false;
                    });
                Assert.AreEqual(string.Empty, t11r.GetColumnError(ds.T11.ValueColumn));

                RXForValidator.AddColumnError(t11r, ds.T11.ValueColumn, "aaa",
                    delegate
                    {
                        return true;
                    });
                Assert.AreEqual("aaa", t11r.GetColumnError(ds.T11.ValueColumn));
            }
        }

        [TestMethod]
        public void AddColumnErrorIfRequireIsNot()
        {
            using (RXDataSet ds = new RXDataSet())
            {
                RXDataSet.T11Row t11r = ds.T11.AddT11Row(1L, "X");
                
                RXForValidator.AddColumnErrorIfRequireIsNot(t11r, ds.T11.ValueColumn);
                Assert.AreEqual(string.Empty, t11r.GetColumnError(ds.T11.ValueColumn));
            }

            using (RXDataSet ds = new RXDataSet())
            {
                RXDataSet.T11Row t11r = ds.T11.AddT11Row(1L, string.Empty);

                RXForValidator.AddColumnErrorIfRequireIsNot(t11r, ds.T11.ValueColumn);
                Assert.AreEqual("Valueは必須入力です。", t11r.GetColumnError(ds.T11.ValueColumn));
            }

            using (RXDataSet ds = new RXDataSet())
            {
                RXDataSet.T11Row t11r = ds.T11.AddT11Row(1L, string.Empty);

                ds.T11.ValueColumn.Caption = "値";
                RXForValidator.AddColumnErrorIfRequireIsNot(t11r, ds.T11.ValueColumn);
                Assert.AreEqual("値は必須入力です。", t11r.GetColumnError(ds.T11.ValueColumn));
            }
        }

        [TestMethod]
        public void AddColumnErrorIfLengthIsNot()
        {
            using (RXDataSet ds = new RXDataSet())
            {
                RXDataSet.T11Row t11r = ds.T11.AddT11Row(1L, "X");

                RXForValidator.AddColumnErrorIfLengthIsNot(t11r, ds.T11.ValueColumn, 2);
                Assert.AreEqual("Valueの長さは2文字でなければなりません。", t11r.GetColumnError(ds.T11.ValueColumn));
            }

            using (RXDataSet ds = new RXDataSet())
            {
                RXDataSet.T11Row t11r = ds.T11.AddT11Row(1L, "XX");

                RXForValidator.AddColumnErrorIfLengthIsNot(t11r, ds.T11.ValueColumn, 2);
                Assert.AreEqual(string.Empty, t11r.GetColumnError(ds.T11.ValueColumn));
            }

            using (RXDataSet ds = new RXDataSet())
            {
                RXDataSet.T11Row t11r = ds.T11.AddT11Row(1L, "XXX");

                RXForValidator.AddColumnErrorIfLengthIsNot(t11r, ds.T11.ValueColumn, 2);
                Assert.AreEqual("Valueの長さは2文字でなければなりません。", t11r.GetColumnError(ds.T11.ValueColumn));
            }
        }
    }
}
