﻿using System;
using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Threading;
using System.Web.Services.Protocols;
using System.Xml;
using TailIsland.RX.Properties;

namespace TailIsland.RX
{
    internal sealed class ErrorMessageGetter
    {
        private Exception exception;

        internal ErrorMessageGetter(Exception exception)
        {
            this.exception = exception;
        }

        internal string GetErrorMessage()
        {
            if (IsRXException(exception))
            {
                return GetRXExceptionMessage(exception);
            }

            if (IsDBConcurrencyException(exception))
            {
                return GetDBConcurrencyErrorMessage();
            }

            if (IsSoapException(exception))
            {
                return GetSoapExceptionDetailMessage(exception);
            }

            if (IsSqlExceptionWithDeadLock(exception))
            {
                return GetDeadLockErrorMessage();
            }

            return GetUnknownErrorMessage();
        }

        private static bool IsRXException(Exception exception)
        {
            return exception is RXException;
        }

        private static bool IsDBConcurrencyException(Exception exception)
        {
            return exception is DBConcurrencyException;
        }

        private static bool IsSoapException(Exception exception)
        {
            return exception is SoapException;
        }

        private static bool IsSqlExceptionWithDeadLock(Exception exception)
        {
            SqlException ex = exception as SqlException;

            return ex != null && ex.ErrorCode == 1205;
        }

        private static string GetRXExceptionMessage(Exception exception)
        {
            return string.IsNullOrEmpty(exception.Message) ? GetUnknownErrorMessage() : exception.Message;
        }

        private static string GetDBConcurrencyErrorMessage()
        {
            return Resources.DBConcurrencyError;
        }

        private static string GetSoapExceptionDetailMessage(Exception exception)
        {
            XmlNode message = FindFirstByLocalName(((SoapException)exception).Detail.ChildNodes, "message");

            if (message == null)
            {
                return GetUnknownErrorMessage();
            }

            return message.InnerText;
        }

        private static XmlNode FindFirstByLocalName(XmlNodeList nodeList, string localName)
        {
            for (int i = 0; i < nodeList.Count; ++i)
            {
                if (nodeList[i].LocalName == localName)
                {
                    return nodeList[i];
                }
            }

            return null;
        }

        private static string GetDeadLockErrorMessage()
        {
            return Resources.DeadLockError;
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Security", "CA2122")]
        private static string GetUnknownErrorMessage()
        {
            return string.Format(Resources.UnknownError, Environment.MachineName, Process.GetCurrentProcess().Id, Thread.CurrentThread.GetHashCode());
        }
    }
}
