﻿using System;
using System.Collections.Generic;
using System.Data;

namespace TailIsland.RX
{
    internal sealed class FillTableAdaptersCaller
    {
        //  取得可能な最大行数。
        private const int maxRecordCount = 500;

        private DataTable dataTable;
        private ICollection<DataTable> lastChildDataTables;
        private string condition;
        private object[] extraParameters;

        internal FillTableAdaptersCaller(DataTable dataTable, ICollection<DataTable> lastChildDataTables, string condition, object[] extraParameters)
        {
            this.dataTable = dataTable;
            this.lastChildDataTables = lastChildDataTables;
            this.condition = condition;
            this.extraParameters = extraParameters;
        }

        internal int CallFillTableAdapters()
        {
            return CallFillTableAdapters(new RelatedDataTablesGetter(dataTable, lastChildDataTables, RelatedDataTablesGetter.Condition.Parent, RelatedDataTablesGetter.SortDirection.ParentIsFirst).GetRelatedDataTables());
        }

        internal int CallFillTableAdaptersWithChildren()
        {
            return CallFillTableAdapters(new RelatedDataTablesGetter(
                dataTable, lastChildDataTables, RelatedDataTablesGetter.Condition.Parent | RelatedDataTablesGetter.Condition.Child | RelatedDataTablesGetter.Condition.ParentOfChild, RelatedDataTablesGetter.SortDirection.ParentIsFirst).GetRelatedDataTables());
        }

        private int CallFillTableAdapters(IEnumerable<DataTable> dataTables)
        {
            int c = 0;

            foreach (DataTable dt in dataTables)
            {
                if (dt == dataTable)
                {
                    c += CallFillTableAdapter(dt, condition, extraParameters);
                }
                else
                {
                    CallFillTableAdapter(dt, condition, extraParameters);
                }
            }

            return c;
        }

        private static int CallFillTableAdapter(DataTable dataTable, string condition, object[] extraParameters)
        {
            int c = new TableAdapterCaller(
                dataTable, string.Format("Fill{0}", string.IsNullOrEmpty(condition) ? string.Empty : condition), extraParameters).CallTableAdapter();

            //  取得したレコード数をチェックします。最大レコード数はメンバ変数maxCountとして定義されています。
            if (c > maxRecordCount)
            {
                throw new ArgumentException(string.Format("取得したレコード数が多すぎます。テーブル{0}から{1}レコードを取得しました。", dataTable.TableName, c));
            }

            return c;
        }
    }
}
