﻿using System;
using System.Data;
using System.Data.SqlClient;
using System.Transactions;

namespace TailIsland.RX
{
    internal sealed class IdentitiesRenumberer
    {
        private DataTable dataTable;

        internal IdentitiesRenumberer(DataTable dataTable)
        {
            this.dataTable = dataTable;
        }

        internal void RenumberIdentities()
        {
            //  実行効率を考慮して、追加された行がない場合は何もしないでリターンします。
            if (dataTable.DataSet.GetChanges(DataRowState.Added) == null)
            {
                return;
            }

            foreach (DataTable dt in new RelatedDataTablesGetter(dataTable, null, RelatedDataTablesGetter.Condition.Child, RelatedDataTablesGetter.SortDirection.ParentIsFirst).GetRelatedDataTables())
            {
                if (!NeedToRenumber(dt))
                {
                    continue;
                }

                foreach (DataRow dr in dt.Rows)
                {
                    if (dr.RowState != DataRowState.Added)
                    {
                        continue;
                    }

                    new IdentitySetter(dr, dr.Table.Columns["Identity"], new NewIdentityGenerator(dr.Table.TableName).GenerateNewIdentity()).SetIdentity();
                }
            }
        }

        private static bool NeedToRenumber(DataTable dataTable)
        {
            return dataTable.PrimaryKey.Length == 1 && dataTable.PrimaryKey[0].ColumnName  == "Identity" && dataTable.PrimaryKey[0].DataType == typeof(long);
        }
    }
}
