﻿using System;
using System.Diagnostics;
using System.Text;

namespace TailIsland.RX
{
    internal sealed class InOutTracer
    {
        private enum InOut { In, Out }

        private Procedure procedure;
        private object[] args;

        internal InOutTracer(Procedure procedure, object[] args)
        {
            this.procedure = procedure;
            this.args = args;
        }

        internal void TraceInOut()
        {
            Trace.TraceInformation(GetInOutTraceMessage(InOut.In, args));

            procedure();

            Trace.TraceInformation(GetInOutTraceMessage(InOut.Out, args));
        }

        private static string GetInOutTraceMessage(InOut inOut, object[] args)
        {
            StringBuilder sb = new StringBuilder();

            sb.Append(string.Format("{0}が{1}しました。", GetCalleeString(), inOut == InOut.In ? "開始" : "終了"));

            if (args != null && args.Length > 0)
            {
                for (int i = 0; i < args.Length; ++i)
                {
                    sb.Append(string.Format("{0}  パラメータ{1}: {2}", Environment.NewLine, i + 1, args[i].ToString()));
                }
            }

            return sb.ToString();
        }

        private static string GetCalleeString()
        {
            foreach (StackFrame sf in new StackTrace().GetFrames())
            {
                if (sf.GetMethod().ReflectedType.Namespace != "TailIsland.RX")
                {
                    return string.Format("{0}.{1}", sf.GetMethod().ReflectedType.Name, sf.GetMethod().Name);
                }
            }

            return string.Empty;
        }
    }
}
