﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Text;
using System.Reflection;

namespace TailIsland.RX
{
    internal sealed class ModifiedAndDeletedRowsLocker
    {
        private DataTable dataTable;

        internal ModifiedAndDeletedRowsLocker(DataTable dataTable)
        {
            this.dataTable = dataTable;
        }

        internal void LockModifiedAndDeletedRows()
        {
            using (DataSet ds = dataTable.DataSet.GetChanges(DataRowState.Modified | DataRowState.Deleted))
            {
                if (ds == null)
                {
                    return;
                }

                using (SqlConnection scon = new SqlConnection(GetConnectionString(dataTable)))
                {
                    scon.Open();

                    foreach (string s in GetLockCommands(
                        new RelatedDataTablesGetter(ds.Tables[dataTable.TableName], null, RelatedDataTablesGetter.Condition.Child, RelatedDataTablesGetter.SortDirection.ParentIsFirst).GetRelatedDataTables()))
                    {
                        using (SqlCommand scom = new SqlCommand(s, scon))
                        {
                            scom.ExecuteNonQuery();
                        }
                    }
                }
            }
        }

        private static string GetConnectionString(DataTable dataTable)
        {
            return (string)GetSettingsPropertyInfo(dataTable).GetValue(GetSettings(dataTable), new object[] { });
        }

        private static PropertyInfo GetSettingsPropertyInfo(DataTable dataTable)
        {
            foreach (PropertyInfo pi in GetSettingsType(dataTable).GetProperties())
            {
                foreach (SpecialSettingAttribute ssa in pi.GetCustomAttributes(typeof(SpecialSettingAttribute), true))
                {
                    if (ssa.SpecialSetting == SpecialSetting.ConnectionString)
                    {
                        return pi;
                    }
                }
            }

            throw new ArgumentException(Properties.Resources.ConnectionStringIsNot, "dataTable");
        }

        private static object GetSettings(DataTable dataTable)
        {
            return GetSettingsType(dataTable).GetProperty("Default").GetValue(null, null);
        }

        private static Type GetSettingsType(DataTable dataTable)
        {
            return Type.GetType(string.Format("{0}.Properties.Settings,{1}", dataTable.GetType().Namespace, dataTable.GetType().Assembly.FullName));
        }

        private static IEnumerable<string> GetLockCommands(IEnumerable<DataTable> dataTables)
        {
            List<DataTable> l = new List<DataTable>(dataTables);

            l.Sort(
                delegate(DataTable x, DataTable y)
                {
                    return -Comparer<string>.Default.Compare(x.TableName, y.TableName);
                });

            foreach (DataTable dt in l)
            {
                foreach (DataRowView drv in new DataView(dt, string.Empty, GetSortPhrase(dt), DataViewRowState.CurrentRows | DataViewRowState.Deleted))
                {
                    yield return string.Format("select {0} from [{1}] with (updlock) where {2}", GetSelectPhrase(drv.Row), drv.Row.Table.TableName, GetWherePhrase(drv.Row));
                }
            }

            //  ロック順は、テーブル名の降順、主キーの降順とします。降順なのは、ユニット・テストを容易にするためです。
        }

        private static string GetSortPhrase(DataTable dataTable)
        {
            StringBuilder sb = new StringBuilder();

            foreach (DataColumn dc in dataTable.PrimaryKey)
            {
                if (sb.Length > 0)
                {
                    sb.Append(", ");
                }
                sb.AppendFormat("[{0}] desc", dc.ColumnName);
            }

            return sb.ToString();

            //  ロック順は、テーブル名の降順、主キーの降順とします。降順なのは、ユニット・テストを容易にするためです。
        }

        private static string GetSelectPhrase(DataRow dataRow)
        {
            StringBuilder sb = new StringBuilder();

            foreach (DataColumn dc in dataRow.Table.PrimaryKey)
            {
                if (sb.Length > 0)
                {
                    sb.Append(", ");
                }
                sb.AppendFormat("[{0}]", dc.ColumnName);
            }

            return sb.ToString();
        }

        private static string GetWherePhrase(DataRow dataRow)
        {
            StringBuilder sb = new StringBuilder();

            foreach (DataColumn dc in dataRow.Table.PrimaryKey)
            {
                if (sb.Length > 0)
                {
                    sb.Append(" and ");
                }
                sb.AppendFormat("[{0}] = {1}{2}{1}", dc.ColumnName, dc.DataType == typeof(string) || dc.DataType == typeof(char) || dc.DataType == typeof(Guid) ? "'" : string.Empty, dataRow[dc, DataRowVersion.Original]);
            }

            return sb.ToString();
        }
    }
}
