﻿using System;
using System.Data;
using System.Data.SqlClient;
using System.Transactions;

namespace TailIsland.RX
{
    internal sealed class NewIdentityGenerator
    {
        private string tableName;

        internal NewIdentityGenerator(string tableName)
        {
            this.tableName = tableName;
        }

        internal long GenerateNewIdentity()
        {
            object identity = null;

            new TransactionExecutor(
                delegate
                {
                    using (SqlConnection scon = new SqlConnection(new RXConnectionStringGetter().GetRXConnectionString()))
                    {
                        scon.Open();

                        using (SqlCommand scom = new SqlCommand("select [MaxIdentity] from [IdentityNumberer] with (updlock) where [TableName] = @tableName", scon))
                        {
                            scom.Parameters.Add("@tableName", SqlDbType.NVarChar, 64).Value = tableName;

                            identity = scom.ExecuteScalar();
                        }

                        if (identity == null)
                        {
                            identity = 1000L;

                            using (SqlCommand scom = new SqlCommand("insert into [IdentityNumberer] values (@tableName, @maxIdentity)", scon))
                            {
                                scom.Parameters.Add("@tableName", SqlDbType.NVarChar, 64).Value = tableName;
                                scom.Parameters.Add("@maxIdentity", SqlDbType.BigInt).Value = identity;

                                scom.ExecuteNonQuery();
                            }
                        }

                        identity = (long)identity + 1L;

                        using (SqlCommand scom = new SqlCommand("update [IdentityNumberer] set [MaxIdentity] = @maxIdentity where [TableName] = @tableName", scon))
                        {
                            scom.Parameters.Add("@maxIdentity", SqlDbType.BigInt).Value = identity;
                            scom.Parameters.Add("@tableName", SqlDbType.NVarChar, 64).Value = tableName;

                            scom.ExecuteNonQuery();
                        }
                    }
                },
                null, TransactionScopeOption.RequiresNew).ExecuteTransaction();

            return (long)identity;
        }
    }
}
