﻿using System;
using System.Data;
using System.Web.Services.Protocols;
using System.Windows.Forms;

namespace TailIsland.RX
{
    public static class RXForForm
    {
        public static void ExecuteEventHandler(Procedure procedure)
        {
            if (procedure == null)
            {
                throw new ArgumentNullException("procedure");
            }

            new FormEventHandlerExecutor(procedure).ExecuteFormEventHandler();
        }

        public static void SetControlMaxLengths(Control control, DataTable dataTable)
        {
            if (control == null)
            {
                throw new ArgumentNullException("control");
            }
            if (dataTable == null)
            {
                throw new ArgumentNullException("dataTable");
            }

            new ControlMaxLengthsSetter(control, dataTable).SetControlMaxLengths();
        }

        public static int GetBindingSourcePosition(BindingSource bindingSource, DataRow currentDataRow)
        {
            if (bindingSource == null)
            {
                throw new ArgumentNullException("bindingSource");
            }
            if (currentDataRow == null)
            {
                throw new ArgumentNullException("currentDataRow");
            }

            return new BindingSourcePositionGetter(bindingSource, currentDataRow).GetBindingSourcePosition();
        }

        public static void DeleteDataRow(DataRow dataRow)
        {
            if (dataRow == null)
            {
                throw new ArgumentNullException("dataRow");
            }

            new DataRowDeleter(dataRow).DeleteDataRow();
        }

        public static void CallValidators(DataTable dataTable, params object[] extraParameters)
        {
            if (dataTable == null)
            {
                throw new ArgumentNullException("dataTable");
            }
            if (extraParameters == null)
            {
                throw new ArgumentNullException("extraParameters");
            }

            new ValidatorsCaller(dataTable, extraParameters).CallValidators();
        }

        public static void InitializeServiceAgent(SoapHttpClientProtocol soapHttpClientProtocol)
        {
            if (soapHttpClientProtocol == null)
            {
                throw new ArgumentNullException("soapHttpClientProtocol");
            }

            new ServiceAgentInitializer(soapHttpClientProtocol).InitializeServiceAgent();
        }

        public static void ClearDataSetErrors(DataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            new DataSetErrorsClearer(dataSet).ClearErrors();
        }

        public static string CollectDataSetErrors(DataSet dataSet)
        {
            if (dataSet == null)
            {
                throw new ArgumentNullException("dataSet");
            }

            return new DataSetErrorsCollector(dataSet).CollectDataSetErrors();
        }
    }
}
