﻿using System;
using System.Collections.Generic;
using System.Data;

namespace TailIsland.RX
{
    public static class RXForValidator
    {
        public static IEnumerable<DataRow> GetAddedOrModifiedRows(DataTable dataTable)
        {
            if (dataTable == null)
            {
                throw new ArgumentNullException("dataTable");
            }

            return new AddedOrModifiedRowsGetter(dataTable).GetAddedOrModifiedRows();
        }

        public static IEnumerable<DataRow> GetAddedRows(DataTable dataTable)
        {
            if (dataTable == null)
            {
                throw new ArgumentNullException("dataTable");
            }

            return new AddedRowsGetter(dataTable).GetAddedRows();
        }

        public static void AddRowError(DataRow dataRow, string errorMessage)
        {
            if (dataRow == null)
            {
                throw new ArgumentNullException("dataRow");
            }

            new RowErrorAdder(dataRow, errorMessage, null).AddRowError();
        }

        public static void AddRowError(DataRow dataRow, string errorMessage, Function<bool> hasError)
        {
            if (dataRow == null)
            {
                throw new ArgumentNullException("dataRow");
            }

            new RowErrorAdder(dataRow, errorMessage, hasError).AddRowError();
        }

        public static void AddColumnError(DataRow dataRow, DataColumn dataColumn, string errorMessage)
        {
            if (dataRow == null)
            {
                throw new ArgumentNullException("dataRow");
            }
            if (dataColumn == null)
            {
                throw new ArgumentNullException("dataColumn");
            }

            new ColumnErrorAdder(dataRow, dataColumn, errorMessage, null).AddColumnError();
        }

        public static void AddColumnError(DataRow dataRow, DataColumn dataColumn, string errorMessage, Function<bool> hasError)
        {
            if (dataRow == null)
            {
                throw new ArgumentNullException("dataRow");
            }
            if (dataColumn == null)
            {
                throw new ArgumentNullException("dataColumn");
            }

            new ColumnErrorAdder(dataRow, dataColumn, errorMessage, hasError).AddColumnError();
        }

        public static void AddColumnErrorIfRequireIsNot(DataRow dataRow, DataColumn dataColumn)
        {
            if (dataRow == null)
            {
                throw new ArgumentNullException("dataRow");
            }
            if (dataColumn == null)
            {
                throw new ArgumentNullException("dataColumn");
            }

            new ColumnErrorAdder(dataRow, dataColumn, string.Format("{0}は必須入力です。", dataColumn.Caption), 
                delegate 
                { 
                    return string.IsNullOrEmpty((string)dataRow[dataColumn]); 
                }
            ).AddColumnError();
        }

        public static void AddColumnErrorIfLengthIsNot(DataRow dataRow, DataColumn dataColumn, int length)
        {
            if (dataRow == null)
            {
                throw new ArgumentNullException("dataRow");
            }
            if (dataColumn == null)
            {
                throw new ArgumentNullException("dataColumn");
            }

            new ColumnErrorAdder(dataRow, dataColumn, string.Format("{0}の長さは{1}文字でなければなりません。", dataColumn.Caption, length),
                delegate
                {
                    return ((string)dataRow[dataColumn]).Length != length;
                }
            ).AddColumnError();
        }
    }
}
