﻿using System;
using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Threading;
using System.Transactions;

namespace TailIsland.RX
{
    public class RdbmsTraceListener : TraceListener
    {
        public override bool IsThreadSafe
        {
            get { return true; }
        }

        public override void TraceData(TraceEventCache eventCache, string source, TraceEventType eventType, int id, object data)
        {
            if (Filter != null && !Filter.ShouldTrace(eventCache, source, eventType, id, null, null, data, null))
            {
                return;
            }

            InsertTrace(eventCache, eventType, Convert.ToString(data));
        }

        public override void TraceData(TraceEventCache eventCache, string source, TraceEventType eventType, int id, params object[] data)
        {
            if (Filter != null && !Filter.ShouldTrace(eventCache, source, eventType, id, null, null, null, data))
            {
                return;
            }

            InsertTrace(eventCache, eventType, StringUtility.CreateCsvLine(data));
        }

        public override void TraceEvent(TraceEventCache eventCache, string source, TraceEventType eventType, int id)
        {
            if (Filter != null && Filter.ShouldTrace(eventCache, source, eventType, id, null, null, null, null))
            {
                return;
            }

            InsertTrace(eventCache, eventType, string.Empty);
        }

        public override void TraceEvent(TraceEventCache eventCache, string source, TraceEventType eventType, int id, string format, params object[] args)
        {
            if (Filter != null && !Filter.ShouldTrace(eventCache, source, eventType, id, format, args, null, null))
            {
                return;
            }

            InsertTrace(eventCache, eventType, string.Format(format, args));
        }

        public override void TraceEvent(TraceEventCache eventCache, string source, TraceEventType eventType, int id, string message)
        {
            if (Filter != null && !Filter.ShouldTrace(eventCache, source, eventType, id, message, null, null, null))
            {
                return;
            }

            InsertTrace(eventCache, eventType, message);
        }

        public override void Write(string message)
        {
            if (Filter != null && !Filter.ShouldTrace(null, string.Empty, TraceEventType.Verbose, 0, message, null, null, null))
            {
                return;
            }

            InsertTrace(null, TraceEventType.Verbose, message);
        }

        public override void WriteLine(string message)
        {
            if (Filter != null && !Filter.ShouldTrace(null, string.Empty, TraceEventType.Verbose, 0, message, null, null, null))
            {
                return;
            }

            InsertTrace(null, TraceEventType.Verbose, message);
        }

        private static void InsertTrace(TraceEventCache eventCache, TraceEventType eventType, string message)
        {
            new TransactionExecutor(
                delegate
                {
                    using (SqlConnection scon = new SqlConnection(new RXConnectionStringGetter().GetRXConnectionString()))
                    {
                        scon.Open();

                        using (SqlCommand scom = new SqlCommand("insert into [Trace] values (@dateTime, @timestamp, @machineName, @processId, @threadId, @type, @message)", scon))
                        {
                            if (eventCache != null)
                            {
                                scom.Parameters.Add("@dateTime", SqlDbType.DateTime).Value = eventCache.DateTime;
                                scom.Parameters.Add("@timestamp", SqlDbType.BigInt).Value = eventCache.Timestamp;
                                scom.Parameters.Add("@machineName", SqlDbType.NVarChar, 64).Value = Environment.MachineName.Length > 64 ? Environment.MachineName.Substring(0, 64) : Environment.MachineName;
                                scom.Parameters.Add("@processId", SqlDbType.Int).Value = eventCache.ProcessId;
                                scom.Parameters.Add("@threadId", SqlDbType.NVarChar, 64).Value = eventCache.ThreadId;
                            }
                            else
                            {
                                scom.Parameters.Add("@dateTime", SqlDbType.DateTime).Value = DBNull.Value;
                                scom.Parameters.Add("@timestamp", SqlDbType.BigInt).Value = DBNull.Value;
                                scom.Parameters.Add("@machineName", SqlDbType.NVarChar, 64).Value = DBNull.Value;
                                scom.Parameters.Add("@processId", SqlDbType.Int).Value = DBNull.Value;
                                scom.Parameters.Add("@threadId", SqlDbType.NVarChar, 64).Value = DBNull.Value;
                            }
                            scom.Parameters.Add("@type", SqlDbType.NVarChar, 64).Value = eventType.ToString();
                            scom.Parameters.Add("@message", SqlDbType.NVarChar, 1024).Value = message != null ? message.Length > 1024 ? message.Substring(0, 1024) : message : string.Empty;

                            scom.ExecuteNonQuery();
                        }
                    }
                },
                null, TransactionScopeOption.RequiresNew).ExecuteTransaction();
        }
    }
}
