﻿using System;
using System.Collections.Generic;
using System.Data;

namespace TailIsland.RX
{
    internal sealed class RelatedDataTablesGetter
    {
        internal enum Condition { Parent = 0x01, Child = 0x02, ParentOfChild = 0x04 }
        internal enum SortDirection { ParentIsFirst, ChildIsFirst };

        private DataTable dataTable;
        private ICollection<DataTable> lastChildDataTables;
        private Condition condition;
        private SortDirection sortDirection;

        private List<DataTable> searchedDataTables = new List<DataTable>();

        internal RelatedDataTablesGetter(DataTable dataTable, ICollection<DataTable> lastChildDataTables, Condition condition, SortDirection sortDirection)
        {
            this.dataTable = dataTable;
            this.lastChildDataTables = lastChildDataTables;
            this.condition = condition;
            this.sortDirection = sortDirection;
        }

        internal IEnumerable<DataTable> GetRelatedDataTables()
        {
            if (sortDirection == SortDirection.ParentIsFirst)
            {
                if ((condition & Condition.Parent) != 0)
                {
                    foreach (DataTable dt in GetParentDataTables(dataTable))
                    {
                        yield return dt;
                    }
                }

                if ((condition & Condition.Child) != 0)
                {
                    foreach (DataTable dt in GetChildDataTables(dataTable))
                    {
                        yield return dt;
                    }
                }
                else
                {
                    yield return dataTable;
                }
            }

            if (sortDirection == SortDirection.ChildIsFirst)
            {
                if ((condition & Condition.Child) != 0)
                {
                    foreach (DataTable dt in GetChildDataTables(dataTable))
                    {
                        yield return dt;
                    }
                }
                else
                {
                    yield return dataTable;
                }

                if ((condition & Condition.Parent) != 0)
                {
                    foreach (DataTable dt in GetParentDataTables(dataTable))
                    {
                        yield return dt;
                    }
                }
            }
        }

        private IEnumerable<DataTable> GetParentDataTables(DataTable targetDataTable)
        {
            foreach (DataRelation dr in targetDataTable.ParentRelations)
            {
                if (dr.ParentTable != targetDataTable)
                {
                    if (!searchedDataTables.Contains(dr.ParentTable))
                    {
                        searchedDataTables.Add(dr.ParentTable);

                        if (sortDirection == SortDirection.ChildIsFirst)
                        {
                            yield return dr.ParentTable;
                        }

                        foreach (DataTable dt in GetParentDataTables(dr.ParentTable))
                        {
                            yield return dt;
                        }

                        if (sortDirection == SortDirection.ParentIsFirst)
                        {
                            yield return dr.ParentTable;
                        }
                    }
                }
            }
        }

        private IEnumerable<DataTable> GetChildDataTables(DataTable targetDataTable)
        {
            if (!searchedDataTables.Contains(targetDataTable))
            {
                searchedDataTables.Add(targetDataTable);

                if (sortDirection == SortDirection.ParentIsFirst)
                {
                    if ((condition & Condition.ParentOfChild) != 0 && targetDataTable != dataTable)
                    {
                        foreach (DataTable dt in GetParentDataTables(targetDataTable))
                        {
                            yield return dt;
                        }
                    }

                    yield return targetDataTable;
                }

                if (lastChildDataTables == null || !lastChildDataTables.Contains(targetDataTable))
                {
                    foreach (DataRelation dr in targetDataTable.ChildRelations)
                    {
                        foreach (DataTable dt in GetChildDataTables(dr.ChildTable))
                        {
                            if (dt != targetDataTable)
                            {
                                yield return dt;
                            }
                        }
                    }
                }

                if (sortDirection == SortDirection.ChildIsFirst)
                {
                    yield return targetDataTable;

                    if ((condition & Condition.ParentOfChild) != 0 && targetDataTable != dataTable)
                    {
                        foreach (DataTable dt in GetParentDataTables(targetDataTable))
                        {
                            yield return dt;
                        }
                    }
                }
            }
        }
    }
}
