﻿using System;
using System.IO;
using System.Text;
using Microsoft.SqlServer.Management.Smo;

namespace TailIsland.RX
{
    internal sealed class SqlScriptExecutor
    {
        private string databaseName;
        private string sqlScriptFileName;

        internal SqlScriptExecutor(string databaseName, string sqlScriptFileName)
        {
            this.databaseName = databaseName;
            this.sqlScriptFileName = sqlScriptFileName;
        }

        internal void ExecuteSqlScript()
        {
            GetDatabase(databaseName).ExecuteNonQuery(GetSqlScript(sqlScriptFileName));
        }

        private static Database GetDatabase(string databaseName)
        {
            return new Database(new Server(@".\SQLExpress"), databaseName);
        }

        private static string GetSqlScript(string sqlScriptFileName)
        {
            using (TextReader tr = new StreamReader(new FileStream(sqlScriptFileName, FileMode.Open, FileAccess.Read), GetEncoding(sqlScriptFileName)))
            {
                return tr.ReadToEnd();
            }
        }

        private static Encoding GetEncoding(string sqlScriptFileName)
        {
            if (IsBigEndianUnicodeFile(sqlScriptFileName))
            {
                return Encoding.BigEndianUnicode;
            }

            if (IsUnicodeFile(sqlScriptFileName))
            {
                return Encoding.Unicode;
            }

            if (IsUtf8File(sqlScriptFileName))
            {
                return Encoding.UTF8;
            }

            return Encoding.Default;
        }

        private static bool IsBigEndianUnicodeFile(string fileName)
        {
            return FileStartsWith(fileName, 0x00fe, 0x00ff);
        }

        private static bool IsUnicodeFile(string fileName)
        {
            return FileStartsWith(fileName, 0x00ff, 0x00fe);
        }

        private static bool IsUtf8File(string fileName)
        {
            return FileStartsWith(fileName, 0x00ef, 0x00bb, 0x00bf);
        }

        private static bool FileStartsWith(string fileName, params int[] values)
        {
            using (Stream s = new FileStream(fileName, FileMode.Open, FileAccess.Read))
            {
                if (s.Length < values.Length)
                {
                    return false;
                }

                foreach (int v in values)
                {
                    if (s.ReadByte() != v)
                    {
                        return false;
                    }
                }

                return true;
            }
        }
    }
}
