﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Reflection;

namespace TailIsland.RX
{
    internal sealed class TableAdapterCaller
    {
        private DataTable dataTable;
        private string methodName;
        private object[] extraParameters;

        internal TableAdapterCaller(DataTable dataTable, string methodName, object[] extraParameters)
        {
            this.dataTable = dataTable;
            this.methodName = methodName;
            this.extraParameters = extraParameters;
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2201")]
        internal int CallTableAdapter()
        {
            try
            {
                return (int)GetTableAdapterMethodInfo(dataTable, methodName).Invoke(GetTableAdapter(dataTable), GetTableAdapterMethodParameters(dataTable, extraParameters));
            }
            catch (NullReferenceException ex)
            {
                throw new NullReferenceException(string.Format("{0} DataTable={1}, MethodName={2}", ex.Message, dataTable.TableName, methodName), ex);
            }
            catch (TargetInvocationException ex)
            {
                throw ex.InnerException;
            }
        }

        private static MethodInfo GetTableAdapterMethodInfo(DataTable dataTable, string methodName)
        {
            //  Update()の場合は、オーバーロードを考慮し、DataTableを引数に取るメソッドを取得します。
            if (methodName == "Update")
            {
                return GetTableAdapterType(dataTable).GetMethod(methodName, new Type[] { dataTable.GetType() });
            }

            return GetTableAdapterType(dataTable).GetMethod(methodName);
        }

        private static object GetTableAdapter(DataTable dataTable)
        {
            return Activator.CreateInstance(GetTableAdapterType(dataTable));
        }

        private static object[] GetTableAdapterMethodParameters(DataTable dataTable, object[] extraParameters)
        {
            List<object> l = new List<object>();

            l.Add(dataTable);
            l.AddRange(extraParameters);

            return l.ToArray();
        }

        private static Type GetTableAdapterType(DataTable dataTable)
        {
            return Type.GetType(string.Format("{0}.{1}TableAdapters.{2}TableAdapter, {3}", dataTable.DataSet.GetType().Namespace, dataTable.DataSet.DataSetName, dataTable.TableName, dataTable.GetType().Assembly.FullName));
        }
    }
}
