﻿using System;
using System.Diagnostics;
using System.IO;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Xml;

namespace TailIsland.RX
{
    public sealed class TraceSoapExtension : SoapExtension, IDisposable
    {
        private Stream oldStream;
        private MemoryStream newStream;

        private bool disposed;

        ~TraceSoapExtension()
        {
            this.Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        private void Dispose(bool disposing)
        {
            if (disposed)
            {
                return;
            }

            disposed = true;

            if (disposing)
            {
                newStream.Dispose();
            }

            //  今回は、開放すべきアンマネージなリソースはありません。
        }

        public override Stream ChainStream(Stream stream)
        {
            oldStream = stream;
            newStream = new MemoryStream();

            return newStream;
        }

        public override object GetInitializer(Type serviceType)
        {
            return null;
        }

        public override object GetInitializer(LogicalMethodInfo methodInfo, SoapExtensionAttribute attribute)
        {
            return null;
        }

        public override void Initialize(object initializer)
        {
            ;
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1062")]
        public override void  ProcessMessage(SoapMessage message)
        {
            switch (message.Stage)
            {
                case SoapMessageStage.BeforeDeserialize:
                    TraceInput();
                    break;

                case SoapMessageStage.AfterDeserialize:
                    break;

                case SoapMessageStage.BeforeSerialize:
                    break;

                case SoapMessageStage.AfterSerialize:
                    TraceOutput();
                    break;
            }
        }

        private void TraceInput()
        {
            Copy(oldStream, newStream);

            TraceStreamContent();
        }

        private void TraceOutput()
        {
            TraceStreamContent();
            
            Copy(newStream, oldStream);
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Reliability", "CA2000")]
		private static void Copy(Stream from, Stream to) 
		{
            TextReader r = new StreamReader(from);
			TextWriter w = new StreamWriter(to);

            w.WriteLine(r.ReadToEnd());
            w.Flush();
		}

        private void TraceStreamContent()
        {
            newStream.Position = 0;

            XmlDocument d = new XmlDocument();
            d.Load(newStream);

            Trace.TraceInformation(d.OuterXml);

            newStream.Position = 0;
        }
    }
}
