﻿using System;
using System.Data;
using System.Transactions;

using IsolationLevel = System.Transactions.IsolationLevel;

namespace TailIsland.RX
{
    internal sealed class TransactionExecutor
    {
        private Procedure transactionProcedure;
        private DataSet consistentFlagDataSet;
        private TransactionScopeOption transactionScopeOption;

        internal TransactionExecutor(Procedure transactionProcedure, DataSet consistentFlagDataSet, TransactionScopeOption transactionScopeOption)
        {
            this.transactionProcedure = transactionProcedure;
            this.consistentFlagDataSet = consistentFlagDataSet;
            this.transactionScopeOption = transactionScopeOption;
        }

        internal void ExecuteTransaction()
        {
            //  トランザクションがアボートされていると例外が出てしまいますので、アボートされていないことをチェックします。
            if (transactionScopeOption != TransactionScopeOption.Required || !IsTransactionAborted())
            {
                using (TransactionScope ts = new TransactionScope(transactionScopeOption, GetAdequateTransactionOptions()))
                {
                    transactionProcedure();

                    //  トランザクションがアボートされていると例外が出てしまいますので、アボートされていないことをチェックします。
                    if (!IsTransactionAborted())
                    {
                        if (consistentFlagDataSet == null || !consistentFlagDataSet.HasErrors)
                        {
                            ts.Complete();
                        }
                    }
                }
            }
        }

        private static TransactionOptions GetAdequateTransactionOptions()
        {
            TransactionOptions tos = new TransactionOptions();

            //  分離レベルをReadCommitted、タイムアウトを60秒に設定します。
            tos.IsolationLevel = IsolationLevel.ReadCommitted;
            tos.Timeout = new TimeSpan(0, 1, 0);

            return tos;
        }

        private static bool IsTransactionAborted()
        {
            return Transaction.Current != null && Transaction.Current.TransactionInformation.Status == TransactionStatus.Aborted;
        }
    }
}
