﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Reflection;

namespace TailIsland.RX
{
    internal sealed class ValidatorsCaller
    {
        private DataTable dataTable;
        private object[] extraParameters;

        internal ValidatorsCaller(DataTable dataTable, params object[] extraParameters)
        {
            this.dataTable = dataTable;
            this.extraParameters = extraParameters;
        }

        internal void CallValidators()
        {
            new DataSetErrorsClearer(dataTable.DataSet).ClearErrors();

            foreach (DataTable dt in new RelatedDataTablesGetter(dataTable, null, RelatedDataTablesGetter.Condition.Child, RelatedDataTablesGetter.SortDirection.ParentIsFirst).GetRelatedDataTables())
            {
                if (dt.GetChanges() != null)
                {
                    CallValidator(dt, extraParameters);
                }
            }
        }

        private static void CallValidator(DataTable dataTable, object[] extraParameters)
        {
            GetValidatorMethodInfo(dataTable).Invoke(null, GetValidatorMethodParameters(dataTable, extraParameters));
        }

        private static MethodInfo GetValidatorMethodInfo(DataTable dataTable)
        {
            return GetValidatorType(dataTable).GetMethod("Validate", BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic);
        }

        private static Type GetValidatorType(DataTable dataTable)
        {
            return Type.GetType(string.Format("{0}.{1}Validator, {2}", dataTable.DataSet.GetType().Namespace, dataTable.TableName, dataTable.GetType().Assembly.FullName));
        }

        private static object[] GetValidatorMethodParameters(DataTable dataTable, object[] extraParameters)
        {
            List<object> l = new List<object>();

            l.Add(dataTable);
            l.AddRange(extraParameters);

            return l.ToArray();
        }
    }
}
