﻿using System;
using System.Diagnostics;
using System.Threading;
using System.Web;
using System.Web.UI;

namespace TailIsland.RX
{
    internal sealed class WebFormEventHandlerExecutor
    {
        private Procedure procedure;

        internal WebFormEventHandlerExecutor(Procedure procedure)
        {
            this.procedure = procedure;
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031")]
        internal void ExecuteWebFormEventHandler()
        {
            if (!IsValid())
            {
                return;
            }

            try
            {
                procedure();
            }
            catch (ThreadAbortException)
            {
                ;
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.ToString());

                HttpContext.Current.Server.Transfer(string.Format("~/Error.aspx?errorMessage={0}", HttpUtility.UrlEncode(new ErrorMessageGetter(ex).GetErrorMessage())), false);
            }
        }

        private static bool IsValid()
        {
            try
            {
                return ((Page)HttpContext.Current.Handler).IsValid;
            }
            catch (HttpException ex)
            {
                if ((uint)ex.ErrorCode != 0x80004005)
                {
                    throw;
                }

                return true;
            }
        }
    }
}
